// Etoile.cpp: implementation of the Etoile class.
//
//////////////////////////////////////////////////////////////////////

#include "Etoile.h"
#include <string>

#define max(a,b) (((a)>(b))?(a):(b)) 

// Globals for optimisation
char strTmp[20];

static unsigned int HRpasse=0;
static float coef;
static float cmag;
static float rmag;

	int RAh;
	int RAm;
	float RAs;
	bool DEsign;
	int Deg;
	int Arcmin;
	int Arcsec;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Etoile::Etoile()
{
}

Etoile::~Etoile()
{	delete matTransfo;
}

int Etoile::Lire(ifstream & catalogue)
// Lis les infos de l'toile dans le fichier et calcule x,y et z;
{   char ligne[50];
	catalogue.read(ligne,42);
	ligne[41]='\0';

	strncpy(strTmp,ligne,4);	// HR number
	strTmp[4]='\0';
	HR = atoi(strTmp);

	strncpy(Name,ligne+4,10);	// Name (Flamsteed)
	Name[10]='\0';

	strncpy(strTmp,ligne+14,6);	// SAO number
	strTmp[6]='\0';
	SAO = atoi(strTmp);

	strncpy(strTmp,ligne+20,7);	// RaRad
	strTmp[7]='\0';
	RaRad=atof(strTmp);

	strncpy(strTmp,ligne+27,8);	// DecRad
	strTmp[7]='\0';
	DecRad=atof(strTmp);

    strncpy(strTmp,ligne+35,5);	// Mag
	Mag=atof(strTmp);

	SpType=ligne[40];			// SpType

	if (!(HR%910))
	{	char p=37;
		printf("%i%c ",HR/910*10,p);
	}

	// Calc the Cartesian coord with RA and DE
	RADE_to_XYZ(RaRad,DecRad,XYZ);
	XYZ*=RAYON;

	matTransfo=new float[16];	// Used to store the precalc transfos matrix used to draw the star

	glLoadIdentity();			// Init the current matrix

	// Precalcul de la matrice de Positionnement/Rotation pour le dessin
	glTranslatef(XYZ[0],XYZ[1],XYZ[2]);
	glRotatef(RaRad*180/PI,0,1,0);
	glRotatef(DecRad*180/PI,-1,0,0);
	
	glGetFloatv(GL_MODELVIEW_MATRIX , matTransfo);	// Store the matrix


	switch (SpType)			//Couleur en fonction du type spectral
	{	case 'O':	RGB[0]=0.8f;  RGB[1]=1.0f; RGB[2]=1.3f;  break;
		case 'B':	RGB[0]=0.9f;  RGB[1]=1.0f; RGB[2]=1.2f;  break;
		case 'A':	RGB[0]=0.95f; RGB[1]=1.0f; RGB[2]=1.15f; break;
		case 'F':	RGB[0]=1.05f; RGB[1]=1.0f; RGB[2]=1.05f; break;
		case 'G':	RGB[0]=1.3f;  RGB[1]=1.0f; RGB[2]=0.9f;  break;
		case 'K':	RGB[0]=1.15f; RGB[1]=0.95f;RGB[2]=0.8f;  break;
		case 'M':
		case 'C':	RGB[0]=1.3f;  RGB[1]=0.85f;RGB[2]=0.6f;  break;
		case 'S':	RGB[0]=1.5f;  RGB[1]=0.8f; RGB[2]=0.2f;  break;
	}

	return 0;
}




void Etoile::Afficher(float MagLimite)
{	if (Mag>MagLimite) return;
	// Check if in the field of view, if not return
	if (acos((XYZVision[0]*XYZ[0]+XYZVision[1]*XYZ[1]+XYZVision[2]*XYZ[2])/RAYON)>ChampPrecalc)/*((float)Ouv+4)*PI*((float)winW/winH)/360*/ return;

	// Random coef for star twinkling
	coef=(double)rand()/RAND_MAX;

	// Formula 'taken' from the starchart programm
/*`s' size: this argument provides an alternative way of specifying magnitude 
	symbol sizes, using a formula proposed by Andrew Young in Sky and Telescope, 
	March 1990, p. 311, D = D0 * 10^(k*(v0 - v)). He proposes a k of between 0.07 
	and 0.16, with 0.11 as probably the best. If `s' uses 0.1 magnitude quanta as 
	does `m'. The default values for D0, v0, and k are 0.5 (points), the limiting 
	magnitude of the first main map, and 0.11, respectively. These may be altered 
	with the optional arguments to `s', which set D0, v0 and k in that order, e.g.
	`s 0.7 6.6' sets D0 to 0.7 point, and v0 to magnitude 6.6 so that a magnitude 
	6.6 star will be printed as 0.7 point in diameter, and k will remain 0.11. `s 
	0.7 6.6 0.07' will set D0 and v0 as above, and also set k to 0.07. */
	
	// Calculation of the demi-size of the star texture
	// rmag =0.3*pow(10,(0.12*(MagLimite - Mag)))*(pow(Ouv,0.7)/6);
	float L=pow(100,-Mag/5);
	float leMax=max(RGB[0],RGB[2]);
	rmag = sqrt(L/(pow(L,0.66666)+pow(100,1.7/5)))*20*sqrt(leMax)*pow(Ouv/60,0.4);

	cmag = 1;
	
	//if size of star is too small (clignotement) we put it to Ouv/40 --> no more clignotement
	// And we compensate the difference of brighteness with cmag
	if (rmag*42<Ouv) 
	{	cmag=pow(rmag/(Ouv/40),2);
		rmag=Ouv/42;
	}
	
	

	if (rmag*5>Ouv*(1.5-Mag/20)) 
	{	//cmag=pow(rmag/(Ouv/40),2);
		rmag=Ouv/5*(1.5-Mag/20);
	}

	if (rmag*10>Ouv)
	{	rmag=Ouv/10;
	}
	// Calculation of the luminosity
	//cmag=(MagLimite-Mag)/(MagLimite)+0.2+0.15-0.15*exp(-1./Ouv)-(Ouv/400+0.15)*coef-(MagLimite-Mag/2.)*coefciel;//0.5*pow(10,(0.07*(MagLimite - Mag)));
	
	cmag*=(1-coef/3*Ouv/100);

	glColor3fv(RGB/(leMax)*cmag);

	{	glPushMatrix();
		glMultMatrixf(matTransfo);			// recuperation de la matrice calcul au chargement
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2i(1,0);				//Bas Droite
			glVertex3f(rmag,-rmag,0.0f);
			glTexCoord2i(0,0);				//Bas Gauche
			glVertex3f(-rmag,-rmag,0.0f);
			glTexCoord2i(1,1);				//Haut Droit
			glVertex3f(rmag,rmag,0.0f);
			glTexCoord2i(0,1);				//Haut Gauche
			glVertex3f(-rmag,rmag,0.0f);
		glEnd ();
		glPopMatrix();	
	}

}
