// LesPlanetes.cpp: implementation of the LesPlanetes class.
//
//////////////////////////////////////////////////////////////////////

#include "LesPlanetes.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

static Planete * Sun;
static Planete * Mercury;
static Planete * Venus;
static Planete * Earth;
static Planete * Mars;
static Planete * Jupiter;
static Planete * Saturn;
static Planete * Luna;

// Creation initialisation
LesPlanetes::LesPlanetes()
{	Sun = new Planete(SUN);
	Mercury = new Planete(MERCURY);
	Venus = new Planete(VENUS);
	Earth = new Planete(EARTH);
	Mars = new Planete(MARS);
	Jupiter = new Planete(JUPITER);
	Saturn = new Planete(SATURN);
	Luna = new Planete(LUNA);

	(*Sun).Rayon=1392000./(2*UA);
	(*Sun).Colour[0]=1.0;
	(*Sun).Colour[1]=1.0;
	(*Sun).Colour[2]=0.8;
	(*Sun).TexId=40;
	
	(*Mercury).Rayon=4878./(2*UA);
	(*Mercury).Colour[0]=0.9;
	(*Mercury).Colour[1]=0.9;
	(*Mercury).Colour[2]=0.8;
	(*Mercury).TexId=41;
	
	(*Venus).Rayon=12104./(2*UA);
	(*Venus).Colour[0]=0.8;
	(*Venus).Colour[1]=0.8;
	(*Venus).Colour[2]=0.8;
	(*Venus).TexId=42;

	(*Mars).Rayon=6794./(2*UA);
	(*Mars).Colour[0]=1.0;
	(*Mars).Colour[1]=0.5;
	(*Mars).Colour[2]=0.5;
	(*Mars).TexId=44;

	(*Jupiter).Rayon=142796./(2*UA);
	(*Jupiter).Colour[0]=0.7;
	(*Jupiter).Colour[1]=0.7;
	(*Jupiter).Colour[2]=0.6;
	(*Jupiter).TexId=45;

	(*Saturn).Rayon=120660./(2*UA);
	(*Saturn).Colour[0]=0.7;
	(*Saturn).Colour[1]=0.7;
	(*Saturn).Colour[2]=0.6;
	(*Saturn).TexId=46;

	(*Luna).Rayon=3476./(2*UA);
	(*Luna).Colour[0]=1.0;
	(*Luna).Colour[1]=1.0;
	(*Luna).Colour[2]=1.0;
	(*Luna).TexId=49;
}

LesPlanetes::~LesPlanetes()
{	delete Mercury;
	delete Venus;
	delete Earth;
	delete Mars;
	delete Jupiter;
	delete Saturn;
	delete Sun;
	delete Luna;
}

// Calc all the datas for the planets
void LesPlanetes::Calculer(double date, ObsInfo lieu)
{	vec3_t tmp(0,0,0);
	(*Earth).Calculer(date, lieu, tmp);
	vec3_t helioPosObs((*Earth).HelioCoord);
	(*Sun).Calculer(date, lieu, helioPosObs);
	(*Mercury).Calculer(date, lieu, helioPosObs);
	(*Venus).Calculer(date, lieu, helioPosObs);
	(*Mars).Calculer(date, lieu, helioPosObs);
	(*Jupiter).Calculer(date, lieu, helioPosObs);
	(*Saturn).Calculer(date, lieu, helioPosObs);
	(*Luna).Calculer(date, lieu, helioPosObs);
}

// Draw the planets
void LesPlanetes::Afficher()
{	vec3_t coordLight;
	coordLight = (*Sun).GeoCoord;
	
	//light
	float tmp[4] = {0,0,0,1};
	float tmp2[4] = {0.01,0.01,0.01,1};
	float tmp3[4] = {1,1,1,1};
	float tmp4[4] = {0.5,0.5,0.5,1};
	glLightfv(GL_LIGHT1,GL_AMBIENT,tmp3); 
	glLightfv(GL_LIGHT1,GL_DIFFUSE,tmp3);
	glLightfv(GL_LIGHT1,GL_SPECULAR,tmp);

	glDisable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	
	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT ,tmp2);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE ,tmp3);
	glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION ,tmp);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS ,tmp);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR ,tmp);

	(*Luna).Afficher(coordLight);
	(*Mercury).Afficher(coordLight);
	(*Venus).Afficher(coordLight);
	(*Sun).Afficher(coordLight);
//	(*Earth).Afficher(coordLight);
	(*Mars).Afficher(coordLight);
	(*Jupiter).Afficher(coordLight);
	(*Saturn).Afficher(coordLight);
}


// Look for a nebulae by XYZ coords
int LesPlanetes::Rechercher(vec3_t Pos)
{	Pos.Normalize();
	Planete * plusProche;

	float anglePlusProche=0.001;

	Sun->testDistance(Pos,anglePlusProche,plusProche);
	Mercury->testDistance(Pos,anglePlusProche,plusProche);
	Venus->testDistance(Pos,anglePlusProche,plusProche);
	Mars->testDistance(Pos,anglePlusProche,plusProche);
	Jupiter->testDistance(Pos,anglePlusProche,plusProche);
	Saturn->testDistance(Pos,anglePlusProche,plusProche);
	Luna->testDistance(Pos,anglePlusProche,plusProche);
	if (anglePlusProche>0.99995)
	{	Selectionnee=plusProche;
		return 0;
	}
	else return 1;
}
