// Nebuleuse.cpp: implementation of the Nebuleuse class.
//
//////////////////////////////////////////////////////////////////////

#include "Nebuleuse.h"
#include "tga.h"
#include "Affichage.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Nebuleuse::Nebuleuse()
{	posDash = 0x0F0F;
	incLum = rand()%100;
}

Nebuleuse::~Nebuleuse()
{	delete matTransfo;
}

int Nebuleuse::Lire(FILE * catalogue)
// Lis les infos de la nbuleuse dans le fichier et calcule x,y et z;
{	int rahr;
	float ramin;
	int dedeg;
	int demin;
	matTransfo=new float[16];	// Used to store the precalc transfos matrix used to draw the star

	if (fscanf(catalogue,"%u %u %s %s %d %f %d %d %f %f %f %s\n",&NGC,&Messier, Type, Constellation, &rahr, &ramin,&dedeg,&demin,&Mag,&Taille,&Rotation, Name)<12) return 0;//printf("Chargement de M%u %dh%fmin %ddeg%dmin Vmag=%f Taille=%f Rot:%f\n",Messier,rahr,ramin,dedeg,demin,Mag,Taille,Rotation);

	// Replace the "_" with " "
	char * cc;
	cc=strchr(Name,'_');
	while(cc!=NULL)
	{	(*cc)=' ';
		cc=strchr(Name,'_');
	}
	
	// Calc the RA and DE from the datas
	RaRad=RA_en_Rad(rahr,ramin);
	DecRad=DE_en_Rad(dedeg,demin);

	// Calc the Cartesian coord with RA and DE
	RADE_to_XYZ(RaRad,DecRad,XYZ);
	XYZ*=RAYON;

	matTransfo=new float[16];	// Used to store the precalc transfos matrix used to draw the star

	glLoadIdentity();			// Init the current matrix

	// Precalcul de la matrice de Positionnement/Rotation pour le dessin
	glTranslatef(XYZ[0],XYZ[1],XYZ[2]);
	glRotatef(RaRad*180/PI,0,1,0);
	glRotatef(DecRad*180/PI,-1,0,0);
	glRotatef(Rotation,0,0,1);
	glGetFloatv(GL_MODELVIEW_MATRIX , matTransfo);	// Store the matrix

	RayonPrecalc=RAYON*sin(Taille/2/60*PI/180);
	//printf("rayon : %f\n",RayonPrecalc);

	if (Messier>0)		//Load texture for Messiers
	{	TexID=100+Messier;
		int temp = LireTexture();
		return temp;
	}
	return 0;
}


int Nebuleuse::LireTexture()
{	tga_t image;
	char nomFic[100];
	sprintf(nomFic,"../textures/m%d.tga",Messier);
	texIds[TexID] = image.GenId();
	if (image.Load(nomFic, texIds[TexID]) == 0)
	{	printf("Couldn't load texture %s!!\n",nomFic);
		image.Load("../textures/rien.tga", texIds[TexID]);
		return 2;
	}
	return 1;
}

void Nebuleuse::Afficher()
{	// Check if in the field of view, if not return
	if (acos((XYZVision[0]*XYZ[0]+XYZVision[1]*XYZ[1]+XYZVision[2]*XYZ[2])/RAYON)>ChampPrecalc)/*((float)Ouv+4)*PI*((float)winW/winH)/360*/ return;

	glEnable(GL_BLEND);
	glPushMatrix();
	glMultMatrixf(matTransfo);			// recuperation de la matrice calcul au chargement

	glEnable(GL_TEXTURE_2D);
	float cmag=(1-Mag/12);
	glColor3f(cmag,cmag,cmag);
	glBindTexture (GL_TEXTURE_2D, texIds[TexID]);
	
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(1,0);				//Bas Droite
		glVertex3f(RayonPrecalc,-RayonPrecalc,0.0f);
		glTexCoord2i(0,0);				//Bas Gauche
		glVertex3f(-RayonPrecalc,-RayonPrecalc,0.0f);
		glTexCoord2i(1,1);				//Haut Droit
		glVertex3f(RayonPrecalc,RayonPrecalc,0.0f);
		glTexCoord2i(0,1);				//Haut Gauche
		glVertex3f(-RayonPrecalc,RayonPrecalc,0.0f);
	glEnd ();

	
	if (Ouv>sqrt(Taille)*2 && FlagNeb)
	{	incLum++;
		glColor3f(sqrt(Ouv)/10*(0.4+0.2*sin(incLum/10)),sqrt(Ouv)/10*(0.4+0.2*sin(incLum/10)),0.1);
		glBindTexture (GL_TEXTURE_2D, texIds[28]);	
	
		float r=0.05*Ouv;
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2i(1,0);				//Bas Droite
			glVertex3f(r,-r,0.0f);
			glTexCoord2i(0,0);				//Bas Gauche
			glVertex3f(-r,-r,0.0f);
			glTexCoord2i(1,1);				//Haut Droit
			glVertex3f(r,r,0.0f);
			glTexCoord2i(0,1);				//Haut Gauche
			glVertex3f(-r,r,0.0f);
		glEnd ();
	}
	glPopMatrix();	
}
